// Copyright© 2019 HP Development Company, L.P.
// Subject to the Terms of Use on the HP Developer's Portal at https://developers.hp.com/terms
// SPDX-License-Identifier: MIT

package com.hp.daas.analytics.sample;

import java.util.Arrays;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.oauth2.client.OAuth2ClientContext;
import org.springframework.security.oauth2.client.OAuth2RestTemplate;
import org.springframework.security.oauth2.client.resource.OAuth2ProtectedResourceDetails;
import org.springframework.security.oauth2.client.token.grant.code.AuthorizationCodeResourceDetails;
import org.springframework.security.oauth2.config.annotation.web.configuration.EnableOAuth2Client;
import org.springframework.util.StringUtils;

/**
 * This configuration class initialize OAuth2RestTemplate with OpenId Resource
 * Details
 *
 * @author gandhrek Created on Jan 28, 2019
 */
@Configuration
@EnableOAuth2Client
public class OpenIdConnectConfig {

    private static final Logger log = LoggerFactory.getLogger(OpenIdConnectFilter.class);

    /**
     * The "API Key" value generated when you registered your application
     * at {@link <a> https://developers.hp.com </a>}
     */
    @Value("${clientId}")
    private String clientId;

    /**
     * The "Secret Key" value generated when you registered your application
     * at {@link <a> https://developers.hp.com </a>}
     */
    @Value("${clientSecret}")
    private String clientSecret;

    /**
     * HP DaaS OAuth 2.0 authorize API URI
     *
     * For US customer: https://daas.api.hp.com/oauth/v1/authorize
     * For EU customers: https://eu.daas.api.hp.com/oauth/v1/authorize
     *
     * {@link https://developers.hp.com/hp-daas-proactive-management/authenticating-oauth-20 }
     */
    @Value("${userAuthorizationUri}")
    private String userAuthorizationUri;

    /**
     * HP DaaS OAuth 2.0 Token API URI
     *
     * For US customer: https://daas.api.hp.com/oauth/v1/token
     * For EU customers: https://eu.daas.api.hp.com/oauth/v1/token
     *
     * {@link https://developers.hp.com/hp-daas-proactive-management/authenticating-oauth-20 }
     */
    @Value("${accessTokenUri}")
    private String accessTokenUri;

    /**
     * The URI your users will be sent back to after authorization.
     * This value must match one of the defined OAuth 2.0 Redirect URLs in your application configuration.
     *
     */
    @Value("${redirectUri}")
    private String redirectUri;


    /**
     * This method to to create OAuth2ProtectedResourceDetails
     *
     * @return
     */
    @Bean
    public OAuth2ProtectedResourceDetails getOpenIdResourceDetails() {
        final AuthorizationCodeResourceDetails details = new AuthorizationCodeResourceDetails();
        if (StringUtils.isEmpty(clientId)) {
            log.error("clientId should not be null from application properties/ config");
        }

        if (StringUtils.isEmpty(clientSecret)) {
            log.error("clientSecret should not be null from application properties/ config");
        }

        if (StringUtils.isEmpty(accessTokenUri)) {
            log.error("accessTokenUri should not be null from application properties/ config");
        }

        if (StringUtils.isEmpty(userAuthorizationUri)) {
            log.error("userAuthorizationUri should not be null from application properties/ config");
        }

        if (StringUtils.isEmpty(redirectUri)) {
            log.error("redirectUri should not be null from application properties/ config");
        }

        details.setClientId(clientId);
        details.setClientSecret(clientSecret);
        details.setAccessTokenUri(accessTokenUri);
        details.setUserAuthorizationUri(userAuthorizationUri);
        details.setScope(Arrays.asList("Read"));
        details.setPreEstablishedRedirectUri(redirectUri);
        details.setUseCurrentUri(false);
        return details;
    }

    /**
     * this method initialize OAuth2RestTemplate
     *
     * @param clientContext
     * @return
     */
    @Bean
    public OAuth2RestTemplate openIdTemplate( final OAuth2ClientContext clientContext) {
        final OAuth2RestTemplate template = new OAuth2RestTemplate( getOpenIdResourceDetails(), clientContext);
        return template;
    }
}