// Copyright© 2019 HP Development Company, L.P.
// Subject to the Terms of Use on the HP Developer's Portal at https://developers.hp.com/terms
// SPDX-License-Identifier: MIT

package com.hp.daas.analytics.sample;

import java.io.IOException;
import java.util.Arrays;

import javax.servlet.ServletException;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.oauth2.client.OAuth2RestOperations;
import org.springframework.security.oauth2.client.OAuth2RestTemplate;
import org.springframework.security.oauth2.common.OAuth2AccessToken;
import org.springframework.security.oauth2.common.exceptions.OAuth2Exception;
import org.springframework.security.web.authentication.AbstractAuthenticationProcessingFilter;

/**
 * This is application web filter to attempt authentication using OAuth 2.0
 *
 * @author gandhrek
 * Created on Jan 28, 2019
 */
public class OpenIdConnectFilter extends AbstractAuthenticationProcessingFilter {

    private static final Logger log = LoggerFactory.getLogger(OpenIdConnectFilter.class);

    // OAuth2 RestTemplate
    public OAuth2RestOperations restTemplate;

    /**
     * Constructor
     *
     * @param defaultFilterProcessesUrl
     */
    public OpenIdConnectFilter(String defaultFilterProcessesUrl) {
        super(defaultFilterProcessesUrl);
        setAuthenticationManager(new NoopAuthenticationManager());
    }

    /*
     * (non-Javadoc)
     *
     * @see org.springframework.security.web.authentication.
     * AbstractAuthenticationProcessingFilter#attemptAuthentication(javax.
     * servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    @Override
    public Authentication attemptAuthentication(HttpServletRequest request, HttpServletResponse response) throws AuthenticationException, IOException, ServletException {
        OAuth2AccessToken accessToken;
        try {
            // attempt to get access token using auth code received
            accessToken = restTemplate.getAccessToken();
        } catch (final OAuth2Exception e) {
            log.error("Error while get AccessToken {}", e);
            throw new BadCredentialsException("Could not obtain access token", e);
        }

        try {
            // Setting anonymous user with accessToken as cookie
            log.info("Access Token received : {}", accessToken.toString());
            response.addCookie(createCookie("accessToken", accessToken.toString(),3066, false, null));
            return new UsernamePasswordAuthenticationToken(null, null, Arrays.asList(new SimpleGrantedAuthority("ROLE_USER")));
        } catch (final Exception e) {
            log.error("Error while attempt Authentication {}", e);
            throw new BadCredentialsException("Could not obtain access token", e);
        }
    }

    /**
     * Method to set OAuth2 RestTemplate
     *
     * @param restTemplate2
     */
    public void setRestTemplate(OAuth2RestTemplate restTemplate2) {
        restTemplate = restTemplate2;
    }

    private static class NoopAuthenticationManager implements AuthenticationManager {
        /*
         * (non-Javadoc)
         *
         * @see
         * org.springframework.security.authentication.AuthenticationManager#
         * authenticate(org.springframework.security.core.Authentication)
         */
        @Override
        public Authentication authenticate(Authentication authentication) throws AuthenticationException {
            throw new UnsupportedOperationException("No authentication should be done with this AuthenticationManager");
        }
    }

    /**
     * Helper to create a cookie with specified domain
     *
     * @param cookieName
     * @param cookieValue
     * @param expiryInSecs
     * @param httpOnly
     * @param domain
     * @return
     */
    public static Cookie createCookie(String cookieName, String cookieValue, int expiryInSecs, boolean httpOnly, String domain) {
        Cookie cookie = new Cookie(cookieName, cookieValue);
        cookie.setPath("/");
        cookie.setMaxAge(expiryInSecs);
        cookie.setHttpOnly(httpOnly);
        cookie.setSecure(false);
        if (domain != null && !domain.isEmpty()) {
            cookie.setDomain(domain);
        }
        return cookie;
    }
}
